@@ -72,6 +72,7 @@
 #define HAVE_SYS_IOCTL_H 1
 /* #undef HAVE_SYS_MALLOC_H */
 #define HAVE_SYS_MMAN_H 1
+/* #undef HAVE_SYS_MNTENT_H */
 /* #undef HAVE_SYS_NDIR_H */
 /* #undef HAVE_SYS_PTE_H */
 /* #undef HAVE_SYS_PTEM_H */
@@ -126,7 +127,6 @@
 /* Functions we may want to use. */
 #define HAVE_ACCEPT4 1
 #define HAVE_ACCESS 1
-#define HAVE_ALARM 1
 #define HAVE_ALLOCA 1
 /* #undef HAVE_BFILL */
 #define HAVE_INDEX 1
@@ -232,7 +232,6 @@
 #define HAVE_STRTOUL 1
 #define HAVE_STRTOULL 1
 /* #undef HAVE_TELL */
-/* #undef HAVE_THR_SETCONCURRENCY */
 /* #undef HAVE_THR_YIELD */
 #define HAVE_TIME 1
 #define HAVE_TIMES 1
@@ -255,6 +254,7 @@
 /* #undef HAVE_SOCKADDR_IN6_SIN6_LEN */
 #define STRUCT_TIMESPEC_HAS_TV_SEC 1
 #define STRUCT_TIMESPEC_HAS_TV_NSEC 1
+#define STRUCT_TM_HAS_TM_GMTOFF 1
 
 /* this means that valgrind headers and macros are available */
 /* #undef HAVE_VALGRIND_MEMCHECK_H */
@@ -377,14 +377,13 @@
 /* #undef HAVE_SVR3_SIGNALS */
 /* #undef HAVE_V7_SIGNALS */
 #define HAVE_ERR_remove_thread_state 1
-#define HAVE_X509_check_host 1
+/* #undef HAVE_X509_check_host */
 
 /* #undef HAVE_SOLARIS_STYLE_GETHOST */
 
 #define HAVE_GCC_C11_ATOMICS 1
 /* #undef HAVE_SOLARIS_ATOMIC */
 /* #undef NO_FCNTL_NONBLOCK */
-#define NO_ALARM 1
 
 /* #undef _LARGE_FILES */
 #define _LARGEFILE_SOURCE 1
@@ -499,6 +498,8 @@
 #define HAVE_COMPRESS 1
 #define HAVE_EncryptAes128Ctr 1
 #define HAVE_EncryptAes128Gcm 1
+#define HAVE_des 1
+#define HAVE_hkdf 1
 
 /*
   Stuff that always need to be defined (compile breaks without it)
@@ -530,17 +531,17 @@
 /* #undef SO_EXT */
 
 #define MYSQL_VERSION_MAJOR 11
-#define MYSQL_VERSION_MINOR 2
+#define MYSQL_VERSION_MINOR 5
 #define MYSQL_VERSION_PATCH 2
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 11.2.2"
+#define PACKAGE_STRING "MySQL Server 11.5.2"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "11.2.2"
-#define VERSION "11.2.2"
+#define PACKAGE_VERSION "11.5.2"
+#define VERSION "11.5.2"
 #define PROTOCOL_VERSION 10
 #define PCRE2_CODE_UNIT_WIDTH 8
 
